/*
 CURSOR.H

 License CC0 PUBLIC DOMAIN

 To the extent possible under law, Mark J. Olesen has waived all copyright
 and related or neighboring rights to FDOSTUI Library. This work is published
 from: United States.
*/
#ifndef __cursor_h__

#define CURSOR_MASK_HIDE                0x20

enum cursor_mode
{
    CURSOR_OVERWRITE                    =0,
    CURSOR_INSERT                       =1
};

#ifdef __cplusplus
extern "C" {
#endif

extern enum cursor_mode                 _cursor_mode;

extern void
cursor_set_type(
    char const                          i_scan_start,
    char const                          i_scan_end);

#pragma aux cursor_set_type= \
    "mov ah, 01h"\
    "int 010h" \
    modify [ AH ] \
    parm [CH] [CL] ;

extern void
cursor_get_position(
    char*const                          o_scan_start,
    char*const                          o_scan_end,
    char*const                          o_row,
    char*const                          o_column);

#pragma aux cursor_get_position=\
    "push bx"\
    "push ax"\
    "mov ah, 03h"\
    "int 010h"\
    "pop bx"\
    "mov [byte ptr bx], ch"\
    "pop bx"\
    "mov [byte ptr bx], cl"\
    "mov [byte ptr ds:si], dh"\
    "mov [byte ptr ds:di], dl"\
    modify [ AH ] \
    parm [AX] [BX] [SI] [DI];

extern void
cursor_set_position(
    char const                          i_page_number,
    char const                          i_row,
    char const                          i_column);

#pragma aux cursor_set_position =\
    "mov ah, 02h"\
    "int 010h"\
    modify [ AH ] \
    parm [BH] [DL] [DH];

#ifdef __cplusplus
}
#endif

extern void
cursor_hide();

#define cursor_hide()  cursor_set_type(CURSOR_MASK_HIDE, 0x0)

extern void
cursor_underline();

#define cursor_underline()  cursor_set_type(0x06, 0x07)

extern void
cursor_block();

#define cursor_block()  cursor_set_type(0x00, 0x07)

#define __cursor_h__
#endif
