/*
 SCREEN.H

 License CC0 PUBLIC DOMAIN

 To the extent possible under law, Mark J. Olesen has waived all copyright
 and related or neighboring rights to FDOSTUI Library. This work is published
 from: United States.
*/
#ifndef __screen_h__

enum foreground
{
    fcolor_black                        = 0,
    fcolor_blue                         = 1,
    fcolor_green                        = 2,
    fcolor_cyan                         = 3,
    fcolor_red                          = 4,
    fcolor_magenta                      = 5,
    fcolor_brown                        = 6,
    fcolor_light_gray                   = 7,
    fcolor_gray                         = 8,
    fcolor_light_blue                   = 9,
    fcolor_light_green                  = 10,
    fcolor_light_cyan                   = 11,
    fcolor_light_red                    = 12,
    fcolor_light_magenta                = 13,
    fcolor_yellow                       = 14,
    fcolor_white                        = 15
};

enum background
{
    bcolor_black                        = 0,
    bcolor_blue                         = 1,
    bcolor_green                        = 2,
    bcolor_cyan                         = 3,
    bcolor_red                          = 4,
    bcolor_magenta                      = 5,
    bcolor_brown                        = 6,
    bcolor_light_gray                   = 7
};

#ifdef __cplusplus
extern "C" {
#endif

extern unsigned int const               _video_cols;

extern unsigned int const               _video_rows;

extern int screen_init();

extern void screen_deinit();

extern int screen_set25lines();

extern int screen_set50lines();

extern void 
screen_fill(
    unsigned char const                 i_char,
    enum foreground const               i_fcolor,
    enum background const               i_bcolor);

extern void 
screen_fill_area(
    int const                           i_col,
    int const                           i_row,
    unsigned int const                  i_columns,
    unsigned int const                  i_rows,
    unsigned char const                 i_char,
    enum foreground const               i_fcolor,
    enum background const               i_bcolor);

extern void 
screen_putc(
    int const                           i_col,
    int const                           i_row,
    unsigned char const                 i_char,
    unsigned int const                  i_repeat_count,
    enum foreground const               i_fcolor,
    enum background const               i_bcolor);

extern void 
screen_putvc(
    int const                           i_col,
    int const                           i_row,
    unsigned char const                 i_char,
    unsigned int const                  i_repeat_count,
    enum foreground const               i_fcolor,
    enum background const               i_bcolor);

extern void 
screen_puts(
    int const                           i_col,
    int const                           i_row,
    unsigned char const*                i_string,
    unsigned int const                  i_length,
    enum foreground const               i_fcolor,
    enum background const               i_bcolor);

extern void 
screen_set_color(
    int const                           i_col,
    int const                           i_row,
    unsigned int const                  i_repeat_count,
    enum foreground const               i_fcolor,
    enum background const               i_bcolor);

extern int 
screen_getc(
    int const                           i_col,
    int const                           i_row);

extern short int far*  
screen_read(
    short int far*                      o_block,
    unsigned int const                  i_columns,
    int const                           i_col,
    int const                           i_row);

extern short int far* 
screen_write(
    int const                           i_col,
    int const                           i_row,
    short int const far *               i_block,
    unsigned int const                  i_columns);

#ifdef __cplusplus
}
#endif

#define __screen_h__
#endif
