/*
 SCROLLER.CPP

 License CC0 PUBLIC DOMAIN

 To the extent possible under law, Mark J. Olesen has waived all copyright 
 and related or neighboring rights to FDOSTUI Library. This work is published 
 from: United States.
*/
#include "scroller.hpp"
#include "wm.hpp"
#include "skin.h"
#include "draw.h"
#include "matrix.h"

static void
scroller::horizontal_cb(
    widget const*                       i_widget,
    void*                               io_user_data)
{
    scroller*                           l_scroller;

    l_scroller= reinterpret_cast<scroller*>(io_user_data);
    (*l_scroller).scroll_internal();

    return;
}

static void
scroller::vertical_cb(
    widget const*                       i_widget,
    void*                               io_user_data)
{
    scroller*                           l_scroller;

    l_scroller= reinterpret_cast<scroller*>(io_user_data);
    (*l_scroller).scroll_internal();

    return;
}

scroller::scroller(
    int const                           i_pos_x,
    int const                           i_pos_y,
    unsigned int const                  i_len_x,
    unsigned int const                  i_len_y) :
    group(i_pos_x, i_pos_y, i_len_x, i_len_y),
    m_horizontal(i_pos_x, i_pos_y+i_len_y-1, i_len_x-1, 1),
    m_vertical(i_pos_x+i_len_x-1, i_pos_y, 1, i_len_y-1),
    m_scroll_pos_x(0),
    m_scroll_pos_y(0)
{
    m_horizontal.set_debounce(true);
    m_horizontal.set_orientation(slider::HORIZONTAL);
    m_horizontal.set_range(0, 0);
    m_horizontal.set_user_data(static_cast<void*>(this));
    m_horizontal.set_signal_value(horizontal_cb);
    m_vertical.set_debounce(true);
    m_vertical.set_orientation(slider::VERTICAL);
    m_vertical.set_range(0, 0);
    m_vertical.set_user_data(static_cast<void*>(this));
    m_vertical.set_signal_value(vertical_cb);
    return;
}

scroller::~scroller()
{
    return;
}

void
scroller::draw() const
{

    draw_fill_area(
        m_box.m_pos_x,
        m_box.m_pos_y,
        m_box.m_len_x,
        m_box.m_len_y,
        0,
        _skin_window.m_fcolor,
        _skin_window.m_bcolor);

    group::draw();

    m_horizontal.draw();
    m_vertical.draw();

    return;
}

enum event_response
scroller::event_key(
    struct event_key const&             i_event)
{
    enum event_response                 l_response;

    l_response= group::event_key(i_event);

    return l_response;
}

enum event_response
scroller::event_mouse(
    struct event_mouse const&           i_event)
{
    int                                 l_hit;
    enum event_response                 l_response;

    l_response= RESPONSE_NONE;

    do
    {

        l_hit= m_horizontal.contains(i_event.m_pos_x, i_event.m_pos_y);

        if (l_hit)
        {
            l_response= m_horizontal.event_mouse(i_event);
            break;
        }

        l_hit= m_vertical.contains(i_event.m_pos_x, i_event.m_pos_y);

        if (l_hit)
        {
            l_response= m_vertical.event_mouse(i_event);
            break;
        }

        l_response= group::event_mouse(i_event);

    }while(0);

    return l_response;
}

void
scroller::scroll_internal()
{
    int                                 l_pos_x;
    int                                 l_pos_y;
    int                                 l_delta_x;
    int                                 l_delta_y;
    size_t                              l_slot;
    widget*                             l_widget;

    l_pos_x= static_cast<int>(m_horizontal.get_value());
    l_pos_y= static_cast<int>(m_vertical.get_value());

    l_delta_x= m_scroll_pos_x - l_pos_x;
    l_delta_y= m_scroll_pos_y - l_pos_y;

    do
    {

        if (0 == l_delta_x && 0 == l_delta_y)
        {
            break;
        }

        m_scroll_pos_x= l_pos_x;
        m_scroll_pos_y= l_pos_y;

        for (l_slot= 0; m_array.m_taken > l_slot; l_slot++)
        {
            l_widget= reinterpret_cast<widget *>(m_array.m_base[l_slot]);
            l_pos_x= (*l_widget).get_pos_x() + l_delta_x;
            l_pos_y= (*l_widget).get_pos_y() + l_delta_y;
            (*l_widget).move(l_pos_x, l_pos_y);
        }

        set_damage(DAMAGE_SCROLL);
        wm_draw_widget(this);

    }while(0);

    return;
}

void
scroller::set_scroll_bounds()
{
    struct box                          l_box;
    int                                 l_max_x;
    int                                 l_max_y;
    int                                 l_min_x;
    int                                 l_min_y;
    int                                 l_top;
    size_t                              l_slot;
    widget const*                       l_widget;

    l_min_x= 0;
    l_max_x= 0;
    l_min_y= 0;
    l_max_y= 0;

    for (l_slot= 0; m_array.m_taken > l_slot; l_slot++)
    {
        l_widget= reinterpret_cast<widget *>(m_array.m_base[l_slot]);
        (*l_widget).get_box(l_box);

        if (l_min_x > l_box.m_pos_x)
        {
            l_min_x= l_box.m_pos_x;
        }

        l_top= l_box.m_pos_x + l_box.m_len_x;
        if (l_max_x < l_top)
        {
            l_max_x= l_top;
        }

        if (l_min_y > l_box.m_pos_y)
        {
            l_min_y= l_box.m_pos_y;
        }

        l_top= l_box.m_pos_y + l_box.m_len_y;
        if (l_max_y < l_top)
        {
            l_max_y= l_top;
        }
    }

    l_max_x-= (m_box.m_len_x - 2);

    if (0 > l_max_x)
    {
        l_max_x= 0;
    }

    m_horizontal.set_range(l_min_x, l_max_x);

    l_max_y-= (m_box.m_len_y - 2);

    if (0 > l_max_y)
    {
        l_max_y= 0;
    }

    m_vertical.set_range(l_max_y, l_min_y);

    return;
}
