/*
 WM.CPP

 License CC0 PUBLIC DOMAIN

 To the extent possible under law, Mark J. Olesen has waived all copyright 
 and related or neighboring rights to FDOSTUI Library. This work is published 
 from: United States.
*/
#include "wm.hpp"
#include "wmhelper.hpp"
#include "hw.h"
#include "mouse.h"
#include "cursor.h"
#include "event.h"
#include "array.h"
#include "screen.h"
#include "clip.h"
#include "matrix.h"
#include "screen.h"
#include <sys\types.h>
#include <dos.h>
#include <direct.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

#define ARRAY_GROWBY                    30
wmhelper                                _wm_helper;
static window*                          _wm_grab;
static window*                          _wm_focused;
static struct array                     _wm_array;
static enum wm_error                    _wm_quit_flag;

static unsigned int                     _saved_drive;
static char                             _saved_directory[PATH_MAX+1];

int __far _int24h_handler(
    unsigned int                        i_device_error,
    unsigned int                        i_error_code,
    unsigned __far *                    i_device_header)
{
    return _HARDERR_FAIL;
}

extern enum wm_error
wm_init()
{
    enum wm_error                       l_exit;
    int                                 l_rc;

    l_exit= ERROR_NONE;
    _wm_quit_flag= ERROR_NONE;
    _wm_grab= reinterpret_cast<window*>(0);
    _wm_focused= reinterpret_cast<window*>(0);
    array_assign(&_wm_array, ARRAY_GROWBY);

    _dos_getdrive(&_saved_drive);
    _getdcwd(_saved_drive, _saved_directory, sizeof(_saved_directory));

    do
    {

        cursor_hide();

        l_rc= screen_init();

        if (l_rc)
        {
            l_exit= ERROR_INIT_SCREEN;
            break;
        }

        screen_fill(0, fcolor_black, bcolor_black);

        l_rc= mouse_init();

        if (l_rc)
        {
            l_exit= ERROR_INIT_MOUSE;
            break;
        }

        hw_detect_drives();

        mouse_show();

        _harderr(_int24h_handler);

    }while(0);

    return l_exit;
}

extern void
wm_deinit()
{
    unsigned int                        l_total;

    array_discharge(&_wm_array);
    mouse_deinit();
    screen_deinit();

    _dos_setdrive(_saved_drive, &l_total);
    _chdir(_saved_directory);

    return;
}

extern void
wm_quit(
    enum wm_error const                 i_reason)
{
    _wm_quit_flag= i_reason;
    return;
}

extern void
wm_draw(
    window *const                       i_window)
{
    window *                            l_window;
    unsigned int                        l_slot;

    mouse_hide();
    screen_fill(0, fcolor_white, bcolor_black);

    for (l_slot= 0; _wm_array.m_taken > l_slot; l_slot++)
    {
        l_window= reinterpret_cast<window*>(_wm_array.m_base[l_slot]);
        if (i_window != l_window)
        {
            (*l_window).set_damage(widget::DAMAGE_ALL);
            (*l_window).draw();
            (*l_window).set_damage();
        }
    }

    (*i_window).draw();
    (*i_window).set_damage();

    mouse_show();

    return;
}

extern void
wm_widget_translate_screen(
    int*const                           o_pos_x,
    int*const                           o_pos_y,
    widget const*const                  i_widget)
{
    group const*                        l_parent;

    (*o_pos_x)= (*i_widget).get_pos_x();
    (*o_pos_y)= (*i_widget).get_pos_y();

    do
    {

        l_parent= (*i_widget).get_parent();

        do
        {

            if (0 == l_parent)
            {
                break;
            }

            (*o_pos_x)+= (*l_parent).get_pos_x()+1;
            (*o_pos_y)+= (*l_parent).get_pos_y()+1;

            l_parent= (*l_parent).get_parent();

        }while(1);

    }while(0);

    return;
}

extern void
wm_draw_widget(
    widget *const                       i_widget)
{
    group const*                        l_parent;
    struct box                          l_clip;

    do
    {

        if (false == (*i_widget).get_visible())
        {
            break;
        }

        l_parent= (*i_widget).get_parent();

        if (0 == l_parent)
        {
            (*i_widget).draw();
            break;
        }

        (*l_parent).get_box(l_clip);

        matrix_push();

        do
        {

            matrix_translate((*l_parent).get_pos_x()+1, (*l_parent).get_pos_y()+1);
            l_clip.m_len_x-= 2;
            l_clip.m_len_y-= 2;

            l_parent= (*l_parent).get_parent();

            if (0 == l_parent)
            {
                break;
            }

        }while(1);

        l_clip.m_pos_x= 0;
        l_clip.m_pos_y= 0;
        matrix_transform(&l_clip.m_pos_x, &l_clip.m_pos_y);
        clip_push(&l_clip);

        (*i_widget).draw();

        clip_pop();
        matrix_pop();

        (*i_widget).set_damage();

    }while(0);

    return;
}

extern void
wm_register_window(
    window*const                        i_window)
{
    int                                 l_found;
    unsigned                            l_slot;
    window*                             l_window;

    l_found= 0;
    for (l_slot= 0; _wm_array.m_taken > l_slot; l_slot++)
    {
        l_window= (window*)_wm_array.m_base[l_slot];
        if (i_window == l_window)
        {
            l_found= 1;
            break;
        }
    }

    if (0 == l_found)
    {
        array_push(&_wm_array, i_window);
        if (_wm_focused)
        {
            (*_wm_focused).focus_leave();
        }
        _wm_focused= i_window;
        (*_wm_focused).focus_enter();
    }

    return;
}

static enum event_response
wm_event_keyboard(
    struct event_key const&             i_key)
{
    enum event_response                 l_response;

    l_response= RESPONSE_NONE;

    do
    {

        if (_wm_helper.capture())
        {
            l_response= _wm_helper.event_key(i_key);
            break;
        }

        if (_wm_grab)
        {
            l_response= (*_wm_grab).event_key(i_key);
            break;
        }

        l_response= _wm_helper.event_key(_wm_focused, i_key);

        if (RESPONSE_NONE != l_response)
        {
            break;
        }

        l_response= (*_wm_focused).event_key(i_key);

    }while(0);

    return l_response;
}

static enum event_response
wm_event_mouse(
    struct event_mouse const&           i_mouse)
{
    enum event_response                 l_response;
    int                                 l_hit;
    size_t                              l_slot;
    window*                             l_window;

    l_response= RESPONSE_NONE;

    do
    {

        if (_wm_helper.capture())
        {
            l_response= _wm_helper.event_mouse(i_mouse);
            break;
        }

        if (_wm_grab)
        {
            l_response= (*_wm_grab).event_mouse(i_mouse);
            break;
        }

        l_hit= (*_wm_focused).contains(i_mouse.m_pos_x, i_mouse.m_pos_y);

        if (l_hit)
        {
            l_response= _wm_helper.event_mouse(_wm_focused, i_mouse);
            if (RESPONSE_NONE != l_response)
            {
                break;
            }
            if (_wm_focused)
            {
                l_response= (*_wm_focused).event_mouse(i_mouse);
            }
            break;
        }

        if (0 == (BUTTON_CLICKED & i_mouse.m_state))
        {
            break;
        }

        for (l_slot= 0; _wm_array.m_taken > l_slot; l_slot++)
        {
            l_window= reinterpret_cast<window*>(_wm_array.m_base[l_slot]);
            if (l_window != _wm_focused)
            {
                l_hit= (*l_window).contains(i_mouse.m_pos_x, i_mouse.m_pos_y);
                if (l_hit && (*l_window).get_can_focus())
                {
                    (*_wm_focused).focus_leave();
                    _wm_focused= l_window;
                    wm_draw(l_window);
                    (*l_window).focus_enter();
                    l_response= _wm_helper.event_mouse(_wm_focused, i_mouse);
                    break;
                }
            }
        }

    }while(0);

    return l_response;
}

static void
wm_close_all()
{
    size_t                              l_slot;
    window const*                       l_window;

    _wm_focused= 0;
    _wm_grab= 0;

    for (l_slot= 0; _wm_array.m_taken > l_slot; l_slot++)
    {
        _wm_focused= reinterpret_cast<window*>
            (_wm_array.m_base[l_slot]);
        (void)(*l_window).close(true);
    }

    _wm_array.m_taken= 0;

    return;
}

static void
wm_unregister_all()
{
    size_t                              l_slot;
    window*                             l_window;

    for (l_slot= 0; _wm_array.m_taken > l_slot; l_slot++)
    {
        l_window= reinterpret_cast<window*>(_wm_array.m_base[l_slot]);
        delete l_window;
    }

    _wm_grab= 0;
    _wm_focused= 0;
    _wm_array.m_taken= 0;

    return;
}

extern enum wm_error
wm_run()
{
    enum wm_error                       l_error;
    int                                 l_exit;
    struct event_key                    l_key;
    struct event_mouse                  l_mouse;

    l_error= ERROR_NONE;

    if (_wm_array.m_taken)
    {
        _wm_focused= reinterpret_cast<window*>
            (_wm_array.m_base[_wm_array.m_taken-1]);
    }

    do
    {

        if (_wm_quit_flag || 0 == _wm_focused)
        {
            break;
        }

        l_exit= wm_poll_keyboard(&l_key);

        if (l_exit)
        {
            if (ASCII_ESC == l_key.m_ascii)
            {
                wm_unregister_window(_wm_focused);
                break;
            }
            wm_event_keyboard(l_key);
        }
        else
        {
            l_exit= wm_poll_mouse(&l_mouse);
            if (l_exit)
            {
                wm_event_mouse(l_mouse);
            }
        }

    }while(1);

    wm_close_all();
    wm_unregister_all();

    return l_error;
}

extern enum event_response
wm_run_modal(
    widget&                             i_widget)
{
    enum event_response                 l_response;
    struct event_mouse                  l_mouse;
    struct event_key                    l_key;
    int                                 l_exit;

    i_widget.focus_enter();

    do
    {

        l_response= RESPONSE_NONE;

        l_exit= wm_poll_keyboard(&l_key);

        if (l_exit)
        {
            if (ASCII_ESC == l_key.m_ascii)
            {
                l_response= RESPONSE_CLOSE;
                break;
            }

            l_response= i_widget.event_key(l_key);
        }
        else
        {
            l_exit= wm_poll_mouse(&l_mouse);

            if (l_exit)
            {
                l_response= i_widget.event_mouse(l_mouse);
            }
        }

        if (l_exit)
        {
            if (
                RESPONSE_NONE != l_response &&
                RESPONSE_HANDLED != l_response)
            {
                break;
            }
        }

    }while(1);

    return l_response;
}

extern window*const
wm_grab(
    window*const                        i_window)
{
    window*                             l_prev;

    l_prev= _wm_grab;
    _wm_grab= i_window;

    return l_prev;
}

extern void
wm_unregister_window(
    window*const                        i_window)
{
    unsigned                            l_slot;
    window*                             l_window;

    if (_wm_grab == i_window)
    {
        _wm_grab= 0;
    }

    if (_wm_focused == i_window)
    {
        (*_wm_focused).focus_leave();
        _wm_focused= 0;
    }

    for (l_slot= 0; _wm_array.m_taken > l_slot; l_slot++)
    {
        l_window= (window*)_wm_array.m_base[l_slot];
        if (i_window == l_window)
        {
            array_remove(&_wm_array, l_slot);
            break;
        }
    }

    if (_wm_array.m_taken)
    {
        _wm_focused= reinterpret_cast<window*>
            (_wm_array.m_base[_wm_array.m_taken-1]);
         wm_draw(_wm_focused);
         (*_wm_focused).focus_enter();
    }
    else
    {
        _wm_quit_flag= ERROR_EMPTY_QUEUE;
    }

    return;
}
