/*
 WMPOLL.C

 License CC0 PUBLIC DOMAIN

 To the extent possible under law, Mark J. Olesen has waived all copyright 
 and related or neighboring rights to FDOSTUI Library. This work is published 
 from: United States.
*/
#include "wmpoll.h"
#include "mouse.h"
#include "keyboard.h"

static struct event_key                 _wm_key_unget= {0,0};
static unsigned int                     _wm_key_unget_count= 0;

extern void
wm_poll_keyboard_unget(
    struct event_key const*const        i_event)
{
    _wm_key_unget= (*i_event);
    _wm_key_unget_count= 1;
    return;
}

extern int
wm_poll_keyboard(
    struct event_key*const              o_event)
{
    unsigned char                       l_ascii;
    int                                 l_exit;
    enum scancodes                      l_scan;

    do
    {

        if (_wm_key_unget_count)
        {
            (*o_event)= _wm_key_unget;
            _wm_key_unget_count= 0;
            l_exit= 1;
            break;
        }

        l_exit= keyboard_query();

        if (0 == l_exit)
        {
            break;
        }

        keyboard_read((unsigned char*)&l_scan, &l_ascii);
        (*o_event).m_ascii= l_ascii;
        (*o_event).m_scan= l_scan;

    }while(0);

    return l_exit;
}

extern int
wm_poll_mouse(
    struct event_mouse *const           o_event)
{
    static struct mouse_event           _prev= {0,0,BUTTON_NONE};
    int                                 l_exit;
    struct mouse_event                  l_event;

    do
    {
        l_exit= mouse_get_event(&l_event);

        if (0 == l_exit)
        {
            break;
        }

        (*o_event).m_pos_x= (l_event.m_curs_col / 8);
        (*o_event).m_pos_y= (l_event.m_curs_row / 8);

        if (l_event.m_btn_state)
        {
            if (_prev.m_btn_state)
            {
                (*o_event).m_state= BUTTON_DRAGGING;
            }
            else
            {
                (*o_event).m_state= BUTTON_CLICKED;
            }
        }
        else 
        {
            if (_prev.m_btn_state)
            {
                (*o_event).m_state= BUTTON_RELEASED;
            }
            else
            {
                (*o_event).m_state= BUTTON_NONE;
            }
        }

        _prev= l_event;

    }while(0);

    return l_exit;
}
